@@ -30,6 +30,7 @@
 #include <glib/gi18n.h>
 
 #include "applet.h"
+#include "panel-compatibility.h"
 #include "panel-widget.h"
 #include "panel-util.h"
 #include "panel-profile.h"
@@ -55,6 +56,7 @@ typedef enum {
 	APPLICATIONS_MENU,
 #define DEFAULT_MENU      APPLICATIONS_MENU
 	SETTINGS_MENU,
+	PREFERENCES_MENU,
 	LAST_MENU
 } MenuPathRoot;
 
@@ -66,7 +68,9 @@ typedef struct {
 
 static MenuPathRootItem root_items [] = {
 	{ APPLICATIONS_MENU, "applications", "applications.menu" },
-	{ SETTINGS_MENU,     "settings",     "settings.menu"     }
+	{ PREFERENCES_MENU,  "preferences",  "preferences.menu"  },
+	/* compatibility with GNOME < 2.17.90 */
+	{ SETTINGS_MENU,     "settings",     NULL                }
 };
 
 struct _PanelMenuButtonPrivate {
@@ -891,19 +895,31 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 				   gboolean     exactpos,
 				   const char  *id)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *menu_path;
-	char        *custom_icon;
-	char        *tooltip;
-	gboolean     use_menu_path;
-	gboolean     use_custom_icon;
+	GConfClient  *client;
+	char         *scheme;
+	MenuPathRoot root;
+	const char   *key;
+	char         *menu_path;
+	char         *custom_icon;
+	char         *tooltip;
+	gboolean      use_menu_path;
+	gboolean      use_custom_icon;
 
 	client  = panel_gconf_get_client ();
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
 	menu_path = gconf_client_get_string (client, key, NULL);
 
+	scheme = g_strndup (menu_path, strcspn (menu_path, ":"));
+	root = panel_menu_scheme_to_path_root (scheme);
+	g_free (scheme);
+	if (root == SETTINGS_MENU) {
+		panel_compatiblity_migrate_settings_menu_button (panel_gconf_get_client (),
+								 id);
+		g_free (menu_path);
+		return;
+	}
+
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "custom_icon");
 	custom_icon = gconf_client_get_string (client, key, NULL);
 
@@ -1005,6 +1021,7 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 					   &error);
 		if (error) {
 			g_error_free (error);
+			error = NULL;
 			panel_launch_desktop_file ("gmenu-simple-editor.desktop",
 						   "gmenu-simple-editor",
 						   screen,
@@ -1012,12 +1029,14 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 		}
 
                 if (error) {
-                        panel_error_dialog (screen,
-                                            "cannot_exec_gmenu-simple-editor", TRUE,
-                                            _("Could not execute '%s'"),
-                                            "%s",
-                                            "gmenu-simple-editor",
-                                            error->message);
+			char *primary;
+			primary = g_strdup_printf (_("Could not execute '%s'"),
+						   "gmenu-simple-editor");
+                        panel_error_dialog (NULL, screen,
+                                            "cannot_exec_gmenu-simple-editor",
+					    TRUE,
+					    primary, error->message);
+			g_free (primary);
                         g_error_free (error);
                 }
 	}
